<?php
// Released into the public domain, free for anyone to use for any purpose
// Created by Jeff Bush <jeff@coderforlife.com>
// See www.coderforlife.com/projects/linecount2 for more information


// Files and directories to scan. Will scan all subdirectories as well.
$include = array('.','extras/datagrid/*','extras/zipstream.inc','extras/password/strength.inc','extras/password/*.php','extras/password/create.inc');

// Files and directories to avoid scanning. If a directory is excluded, no subdirectories will be scanned either.
$exclude = array('notes','other','ideas','extras','library/firebuglite','library/cache','library/Persist.js');

// Extensions of files to count the number of lines for
$exts = array('.php', '.inc', '.css', '.js', '.java', '.htm', '.html', '.xml', '.htc', '.sql');

// The directory with files icons
$image_dir = '/ext/';

// The file that contains the mime magic data
$magic_file = '/ext/magic.mgc';




// Everything else doesn't need to be changed
function absolute($f) { //like realpath, but uses the unix-style path format
	return preg_replace('/^[a-zA-Z]:\//','/', str_replace('\\','/',realpath($f)));
}

$root = new Dir(absolute('/'));

ini_set('mime_magic.magicfile', absolute($magic_file));

$files = 0;
$folders = 0;
$size = 0;
$codeSize = 0;
$totalLines = 0;
$codeLines = 0;

abstract class Node {
	public $parent;
	public $name;
	public function __construct($name, $parent = NULL) {
		$this->name = $name;
		$this->parent = $parent;
	}
	public abstract function image();
	public abstract function printStructure($depth = 0);
	public abstract function isDirectory();
	public abstract function isFile();
	public function path() { return $this->parent == null ? $this->name : $this->parent->path().$this->name; }
}

class Dir extends Node {
	private $children = array();
	private $nameToIndex = array();
	public function __construct($name, $parent = NULL) {
		global $file_seperator;
		if (strrpos($name, '/') !== strlen($name)-1) { $name .= '/'; }
		parent::__construct($name, $parent);
	}
	public function image() { return $GLOBALS['image_dir'].'special/folder.png'; }
	public function isDirectory() { return true; }
	public function isFile() { return false; }
	public function printStructure($depth = 0) {
		global $files, $folders, $size, $codeSize, $totalLines, $codeLines;
		if ($depth == 0) {
			$files = $total_size = $total_lines = $code_size = $code_lines = 0;
			echo '<table><thead><tr><th>File</th><th>Files</th><th>Size</th><th>Lines</th></tr></thead><tbody>';
		}
		$folders++;
		$start_files = $files;
		$start_folders = $folders;
		$start_size = $size;
		$start_code_size = $codeSize;
		$start_total_lines = $totalLines;
		$start_code_lines = $codeLines;
		ob_start();
		foreach ($this->children as $c) {
			$c->printStructure($depth+1);
		}
		$out = ob_get_clean();
		$f = $files-$start_files;
		$F = $folders-$start_folders+$f;
		echo '<tr class=folder'.($depth!=0?' style="display:none;"':'').' onclick="toggleFolder(this,'.$F.');">';
		echo '<td><img src="'.$this->image().'" width=32 height=32 alt=folder style="margin-left:'.(20*$depth)."px\"> $this->name</td><td>$f <span style='color:gray'>($F)</span></td>";
		$csz = $codeSize-$start_code_size;
		$sz = $size-$start_size;
		echo "<td><span title=$csz>".bytes(($csz),1)."</span> <span title=$sz style='color:gray'>(".bytes($sz,1).')</span></td>';
		$cl = $codeLines-$start_code_lines;
		$l = $totalLines-$start_total_lines;
		echo "<td>$cl <span style='color:gray'>($l)</span></td></tr>";
		echo $out;
		if ($depth == 0) {
			echo '</tbody></table>';
		}
	}
	private static function nsort($a, $b) { return strcmp($a->name, $b->name); }
	public function condenseAndSort() {
		foreach ($this->children as $c) {
			if ($c->isDirectory())
				$c->condenseAndSort();
		}
		$this->children = array_values(array_filter($this->children));
		$count = count($this->children);
		if ($count == 0) {
			unset($this->parent->children[$this->parent->nameToIndex[rtrim($this->name,'/')]]);
			unset($this->parent->nameToIndex[$this->name]);
		} else if ($count == 1) {
			$c = $this->children[0];
			if ($c->isDirectory()) {
				$this->name .= $c->name;
				$this->children = $c->children;
				$this->nameToIndex = $c->nameToIndex;
				foreach ($this->children as $c)
					$c->parent = $this;
			} else {
				//TODO: what to do here? (directory containing only one file)
			}
		} else {
			usort($this->children, array('Dir','nsort'));
			$this->nameToIndex = array();
			foreach ($this->children as $i=>$c)
				$this->nameToIndex[rtrim($c->name,'/')] = $i;
			ksort($this->nameToIndex);
		}
	}
	public function add($file, $isDirectory) {
		global $file_seperator;
		$path = explode('/', $file);
		if ($path[0] === '') { array_shift($path); }
		return $this->add_($path, $isDirectory);
	}
	private function add_($path, $isDirectory) {
		$next = array_shift($path);
		if (count($path) === 0 && !$isDirectory) {
				$this->nameToIndex[$next] = count($this->children);
			return ($this->children[] = new File($next, $this));
		} else {
			if (isset($this->nameToIndex[$next])) {
				$dir = $this->children[$this->nameToIndex[$next]];
			} else {
				$dir = new Dir($next, $this);
				$this->nameToIndex[$next] = count($this->children);
				$this->children[] = $dir;
			}
			if (count($path) !== 0)
				return $dir->add_($path, $isDirectory);
			return $dir;
		}
	}
}

class File extends Node {
	public function isDirectory() { return false; }
	public function isFile() { return true; }
	public function image() {
		$ext = substr($this->name, strrpos($this->name, '.')+1);
		$file = $GLOBALS['image_dir']."$ext.png";
		if (file_exists($_SERVER['DOCUMENT_ROOT'].$file))
			return $file;
		if (function_exists('finfo_file')) {
			$finfo = finfo_open(FILEINFO_MIME, absolute($GLOBALS['magic_file']));
			$mime = finfo_file($finfo, $this->path());
			finfo_close($finfo);
		} else if (function_exists('mime_content_type')) {
			$mime = mime_content_type($this->path());
		} else { $mime = ''; }
		$basic = substr($mime, 0, strpos($mime, '/'));
		$file = $GLOBALS['image_dir']."special/$basic.png";
		if (file_exists($_SERVER['DOCUMENT_ROOT'].$file))
			return $file;
		error_log('No image could be found to represent ' .$this->path()." ($mime)");
		return $GLOBALS['image_dir'].'special/unknown.png';
	}
	public function printStructure($depth = 0) {
		echo '<tr'.($depth!=0?' style="display:none;"':'').'><td><img src="'.$this->image().'" width=32 height=32 alt="*" style="margin-left:'.(20*$depth).'px"> '.$this->name.'</td><td></td>';
		$lines = $this->countLines();
		echo '<td><span title='.$lines[1].'>'.bytes($lines[1],1).'</span> <span title='.$lines[0].' style="color:gray">('.bytes($lines[0],1).')</span></td>';
		echo "<td>{$lines[3]} <span style='color:gray'>({$lines[2]})</span></td></tr>";
	}
	public function countLines() {
		global $files, $size, $codeSize, $totalLines, $codeLines;
		$filesize = filesize($this->path());
		$files++;
		$text = file_get_contents($this->path());
		$code = $text = preg_replace('/\r\n/',"\n",$text); //convert line endings

    if (!function_exists('remove_comment')) {
      function remove_comment($matches) { //for comments return nothing, for strings returns the string
          return ($matches[0][0]=='#'||$matches[0][0]=='/')?'':$matches[0];
      }
    }
		$code = preg_replace_callback('~"([^\\\\"]*(\\\\[^\n]|))*"|\'([^\\\\\']*(\\\\[^\n]|))*\'|(//|#)[^\n]*$|/\*.*?\*/~ms','remove_comment',$code);
    $code = preg_replace(array('~^\s+~m','~\s+$~m'),'',$code); //remove leading/trailing space on each line
		$code = preg_replace(array('~\n\n+~','~\s\s+~'), array("\n",' '), $code); //remove double spaces and all blank lines
		$code = preg_replace(array('~^\n+~','~\n+$~'), '', $code); //remove blank lines at the beginning and the ending of the file

		$total_size = $filesize;
		$total_lines = substr_count($text, "\n")+1;
		$code_size = strlen($code);
		$code_lines = substr_count($code, "\n")+1;

		$size += $filesize;
		$codeSize += $code_size;
		$totalLines += $total_lines;
		$codeLines += $code_lines;

		return array($total_size, $code_size, $total_lines, $code_lines);
	}
}
?>
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd">
<html lang="en-US"><head><title>Linecount</title>
<script type="text/javascript">
function toggleFolder(r,c) {
	var open = r.nextSibling.style.display=='none';
	r.cells[0].firstChild.src = '<?php echo $image_dir ?>special/folder'+(open?'-open':'')+'.png';
	for (var i = 0, n = r.nextSibling; i < c; i++) {
		n.style.display = open?'':'none';
		n = n.nextSibling;
	}
}
</script>
<style type="text/css">
img { vertical-align: middle; }
div.folder { margin-left: 20px; }
div.folderLink { cursor: pointer; }
table { border-spacing: 0; }
tr.folder { font-weight: bold; cursor: pointer; }
tbody tr:hover { background: lightblue; }
td { padding: 1px 5px; margin: 0; }
</style>
</head>
<body>
<?php

foreach ($exclude as &$exc) {
	$exc = absolute($exc);
}

function processFiles($files) {
	global $root, $exclude;
	foreach ($files as $file) {
		$isDir = is_dir($file);
		if (in_array($file, $exclude) || !$isDir && !checkExtension($file)) continue;
		$file = $root->add($file, $isDir);
		if ($isDir)
			processFiles(glob($file->path().'*'));
	}
}
foreach ($include as $inc) {
	processFiles(array_map('absolute', glob($inc)));
}

$root->condenseAndSort();
$root->printStructure();

function checkExtension($name) {
	global $exts;
	$len = strlen($name);
	foreach ($exts as $ext)
		if (strrpos($name, $ext) == $len-strlen($ext))
			return true;
	return false;
}

function bytes($bytes, $decimals) {
	if ($bytes < 1024)
		return $bytes.'B';
	if ($bytes < 1048576)
		return round($bytes/1024, $decimals).'KB';
	return round($bytes/(1024*1024), $decimals).'MB';
}

?>
</body></html>